<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use App\Http\Requests\API\Requirement\ClosedRequest;
use App\Http\Requests\API\Requirement\CreateOrUpdateRequest;
use App\Http\Requests\API\Requirement\LinkPlanRequest;
use App\Http\Requests\API\Requirement\BatchCreateRequest;
use App\Http\Resources\API\AssetRequirementResource;
use App\Http\Resources\API\RequirementResource;
use App\Models\Asset;
use App\Models\Enums\RequirementStatus;
use App\Models\Plan;
use App\Models\Requirement;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\DB;
use Illuminate\Validation\Rules\Enum;

class RequirementController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index(Request $request)
    {
        $requirements=Requirement::filter($request->all())->where('company_id',Auth::user()->company_id)->with(['createdBy', 'plan','group','asset'=>function($query){
            $query->with('parent');
        }]) ->simplePaginate();

        return AssetRequirementResource::collection($requirements);
    }

    public function byAssets(Request $request){
        $assetIds = $request->get("assets", []);
        $emptyResponse = $this->success(['data' => []]);
        if (! $assetIds) {
            return $emptyResponse;
        }

        $assets = Asset::query()->where(function ($query) use ($assetIds) {
            foreach ($assetIds as $index => $assetId) {
                $where = $index == 0 ? "where" : "orWhere";
                $query->$where("path", "like", "%,". $assetId . ",%");
            }
        })->pluck("id");
        if ($assets->isEmpty()) {
            return $emptyResponse;
        }

        $requirement=Requirement::query()->with('asset')->whereIn('asset_id',$assets->toArray())->get();
        return AssetRequirementResource::collection($requirement);
    }

//    public function byAsset(Request $request,string $assetId)
//    {
//        $requirements=Requirement::filter($request->all())->with(['createdBy', 'plan','group']) ->where('asset_id', $assetId)->simplePaginate();
//
//        return AssetRequirementResource::collection($requirements);
//    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(CreateOrUpdateRequest $request)
    {
        $requirement = new Requirement();

        $requirement->mergeFillable([
            'company_id', 'created_by',
        ]);

        $requirement->fill([
            ...$request->all(),
            'company_id' => Auth::user()->company_id,
            'created_by' => Auth::id(),
        ]);
        $requirement->save();

        return $this->created();
    }

    /**
     * Display the specified resource.
     */
    public function show(string $id)
    {
        $requirement = Requirement::query()->findOrFail($id);

        return new RequirementResource($requirement);
    }

    /**
     * Update the specified resource in storage.
     */
    public function update(CreateOrUpdateRequest $request, string $id)
    {
        //
        $requiremen = Requirement::findOrFail($id);
        $requiremen->fill($request->all());
        $requiremen->save();
        return $this->noContent();
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy(string $id)
    {
        $requiremen = Requirement::findOrFail($id);

        $requiremen->delete();

        return $this->noContent();
    }

    public function close(ClosedRequest $request,string $id){
        $requiremen = Requirement::findOrFail($id);
        $requiremen->update([
            ...$request->all(),
            'status' =>RequirementStatus::CLOSED->value,
        ]);

         return $this->noContent();
    }

    public function linkPlan(LinkPlanRequest $request, string $planId)
    {
        $plan = Plan::query()->findOrFail($planId);

        Requirement::query()->where("asset_id", $plan->asset_id)
            ->whereIn('id', $request->get("requirement_ids"))
            ->update([
                'plan_id' => $plan->id,
            ]);

        return $this->noContent();
    }

    public function unlinkPlan(LinkPlanRequest $request)
    {
        Requirement::query()->whereIn('id', $request->get("requirement_ids"))
            ->update([
                'plan_id' => null,
            ]);

        return $this->noContent();
    }

    public function batchStore(BatchCreateRequest $request)
    {
        $requirementsData = $request->all();
        $companyId = Auth::user()->company_id;
        $created_by = Auth::id();
        DB::transaction(function () use ($requirementsData,$companyId,$created_by) {
            foreach ($requirementsData as $k => $data) {
                $requirement = new Requirement();
                $requirement->mergeFillable([
                    'company_id', 'created_by',
                ]);
                if($k != 0){
                    $data["requirement_group_id"] = $data["requirement_group_id"] == 'ditto'? $requirementsData[$k-1]["requirement_group_id"]:$data["requirement_group_id"];
                    $data["priority"] = $data["priority"] == 'ditto'? $requirementsData[$k-1]["priority"]:$data["priority"];
                }
                $requirement->fill([
                    ...$data,
                    'company_id' => $companyId,
                    'created_by' =>  $created_by,
                    'status' => RequirementStatus::ACTIVE,
                    'mailto' => [],
                ]);
                $requirement->save();
            }
        });
        return $this->created();
    }
}
